import { IThemeManager } from '@jupyterlab/apputils';
import { ITranslator } from '@jupyterlab/translation';
import { CommandRegistry } from '@lumino/commands';
import { Message } from '@lumino/messaging';
import { Panel } from '@lumino/widgets';
import { IDebugger } from '../../tokens';
/**
 * A data grid that displays variables in a debugger session.
 */
export declare class VariablesBodyGrid extends Panel {
    /**
     * Instantiate a new VariablesBodyGrid.
     *
     * @param options The instantiation options for a VariablesBodyGrid.
     */
    constructor(options: VariablesBodyGrid.IOptions);
    /**
     * The variable filter list.
     */
    get filter(): Set<string>;
    set filter(filter: Set<string>);
    /**
     * The current scope of the variables.
     */
    get scope(): string;
    set scope(scope: string);
    protected commands: CommandRegistry;
    protected model: IDebugger.Model.IVariables;
    protected themeManager: IThemeManager | null | undefined;
    protected translator: ITranslator | undefined;
    /**
     * Load the grid panel implementation and instantiate a grid.
     */
    protected initialize(): Promise<void>;
    /**
     * Wait until actually displaying the grid to trigger initialization.
     */
    protected onBeforeShow(msg: Message): void;
    /**
     * Handle `update-request` messages.
     */
    protected onUpdateRequest(msg: Message): void;
    private _filter;
    private _grid;
    private _pending;
    private _scope;
}
/**
 * A namespace for `VariablesBodyGrid` statics.
 */
export declare namespace VariablesBodyGrid {
    /**
     * Instantiation options for `VariablesBodyGrid`.
     */
    interface IOptions {
        /**
         * The variables model.
         */
        model: IDebugger.Model.IVariables;
        /**
         * The commands registry.
         */
        commands: CommandRegistry;
        /**
         * The optional initial scopes data.
         */
        scopes?: IDebugger.IScope[];
        /**
         * An optional application theme manager to detect theme changes.
         */
        themeManager?: IThemeManager | null;
        /**
         * The application language translator.
         */
        translator?: ITranslator;
    }
}
