"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isES6 = isES6;
exports.isValid = isValid;
exports.tryLoad = tryLoad;
var _debug = _interopRequireDefault(require("debug"));
var _lodash = _interopRequireDefault(require("lodash"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const debug = (0, _debug.default)('verdaccio:plugin:loader:utils');
const MODULE_NOT_FOUND = 'MODULE_NOT_FOUND';
function isValid(plugin) {
  // @ts-expect-error default not relevant
  return _lodash.default.isFunction(plugin) || _lodash.default.isFunction(plugin.default);
}
function isES6(plugin) {
  return Object.keys(plugin).includes('default');
}

/**
 * Requires a module.
 * @param {*} path the module's path
 * @return {Object}
 */
function tryLoad(path, onError) {
  try {
    debug('loading plugin %s', path);
    return require(path);
  } catch (err) {
    if (err.code === MODULE_NOT_FOUND) {
      debug('plugin %s not found', path);
      return null;
    }
    onError({
      err: err.msg
    }, 'error loading plugin @{err}');
    throw err;
  }
}
//# sourceMappingURL=utils.js.map