import { NextFunction } from 'express';
import { pluginUtils } from '@verdaccio/core';
import { Callback, Config, JWTSignOptions, Logger, RemoteUser } from '@verdaccio/types';
import { $RequestExtend, $ResponseExtend } from '../types';
declare class Auth {
    config: Config;
    logger: Logger;
    secret: string;
    plugins: pluginUtils.Auth<Config>[];
    constructor(config: Config);
    private _loadPlugin;
    private _applyDefaultPlugins;
    changePassword(username: string, password: string, // pragma: allowlist secret
    newPassword: string, // pragma: allowlist secret
    cb: Callback): void;
    authenticate(username: string, password: string, cb: Callback): void;
    add_user(user: string, password: string, cb: Callback): void;
    /**
     * Allow user to access a package.
     */
    allow_access({ packageName, packageVersion }: pluginUtils.AuthPluginPackage, user: RemoteUser, callback: Callback): void;
    allow_unpublish({ packageName, packageVersion }: pluginUtils.AuthPluginPackage, user: RemoteUser, callback: Callback): void;
    /**
     * Allow user to publish a package.
     */
    allow_publish({ packageName, packageVersion }: pluginUtils.AuthPluginPackage, user: RemoteUser, callback: Callback): void;
    apiJWTmiddleware(): import("express").RequestHandler<import("express-serve-static-core").ParamsDictionary, any, any, import("qs").ParsedQs, Record<string, any>> | ((req: $RequestExtend, res: $ResponseExtend, _next: NextFunction) => void);
    private _handleJWTAPIMiddleware;
    private _handleAESMiddleware;
    private _isRemoteUserValid;
    /**
     * JWT middleware for WebUI
     */
    webUIJWTmiddleware(): Function;
    jwtEncrypt(user: RemoteUser, signOptions: JWTSignOptions): Promise<string>;
    /**
     * Encrypt a string.
     */
    aesEncrypt(value: string): string | void;
}
export default Auth;
